"""Product block for :class:`Switch` products."""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic_forms.types import strEnum

from gso.products.product_blocks.site import (
    SiteBlock,
    SiteBlockInactive,
    SiteBlockProvisioning,
)
from gso.utils.shared_enums import Vendor
from gso.utils.types.ip_address import PortNumber


class SwitchModel(strEnum):
    """Enumerator for the different types of switches."""

    EX3400 = "EX3400"


class SwitchBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="SwitchBlock",
):
    """A switch that's being currently inactive. See :class:`SwitchBlock`."""

    switch_fqdn: str | None = None
    switch_ts_port: PortNumber | None = None
    switch_site: SiteBlockInactive | None = None
    switch_vendor: Vendor | None = None
    switch_model: SwitchModel | None = None


class SwitchBlockProvisioning(SwitchBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A switch that's being provisioned. See :class:`SwitchBlock`."""

    switch_fqdn: str
    switch_ts_port: PortNumber
    switch_site: SiteBlockProvisioning
    switch_vendor: Vendor
    switch_model: SwitchModel


class SwitchBlock(SwitchBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A switch that's currently deployed in the network."""

    #: The :term:`FQDN` of the switch.
    switch_fqdn: str
    #:  The port of the terminal server that this switch is connected to. Used to offer out of band access.
    switch_ts_port: PortNumber
    #:  The :class:`Site` that this switch resides in. Both physically and computationally.
    switch_site: SiteBlock
    #: The vendor of the switch.
    switch_vendor: Vendor
    #: The model of the switch.
    switch_model: SwitchModel
